/******************************************************************************
 * file:  RuleList.java
 * author:  Dirk Zitterell
 * description:  an Object that contains all the rules for the turing machine
 *****************************************************************************/

import java.util.Vector;
import java.util.Enumeration;

public class RuleList {
    /* variables */
    protected Vector allRules;

    /* empty constructor */
    public RuleList() {
	this.allRules = new Vector();
    }

    /* add a rule */
    public void addRule(Rule rule) {
	allRules.addElement(rule);
    }
    
    /* remove a rule */
    public boolean removeRule(Rule rule) {
	return allRules.removeElement(rule);
    }

    /* returns the the rule, that contains the specified state and read char */
    public Rule getRule(int state, char read) {
	Rule rule;
	Enumeration e = allRules.elements();
	while (e.hasMoreElements()) {
	    rule = (Rule) e.nextElement(); 
	    if ((rule.getState() == state) && (rule.getRead() == read)) {
		return rule;
	    }
	}
	return null;
    }

    /* Returns the index of the specified rule */
    public int indexOf(Rule rule) {
        return allRules.indexOf(rule);
    }
    
    /* returns the number of Rules in this rule list */
    public int getNumberOfRules() {
        return allRules.size();
    }
    
    /* needed in order to implement the interface Enumeration */
    public Enumeration elements() {
	return new RuleListEnumeration(this);
    }

    /* returns a String representing this rule list */
    public String toString() {
	String str = "";
	Enumeration e = this.elements();
	while (e.hasMoreElements()) {
	    str += e.nextElement();
	}
	return str;
    }
}

class RuleListEnumeration implements Enumeration {

    /* variables */
    Vector allRules;
    Enumeration e;
    
    public RuleListEnumeration(RuleList ruleList) {
	this.allRules = ruleList.allRules;
	this.e = this.allRules.elements();
    }

    public boolean hasMoreElements() {
	return e.hasMoreElements();
    }
    
    public Object nextElement() {
	return e.nextElement();
    }
}
