/******************************************************************************
 * file:  Tape.java
 * author:  Dirk Zitterell
 * description:  an object of this class  represents the turing machine's tape
 *****************************************************************************/


import java.util.Enumeration;


public class Tape {
    
    /* variables */
    protected TapeNode leftEnd;
    protected TapeNode rightEnd;
    protected TapeNode current;
    protected boolean reversed;
        
    /* constructor */
    public Tape() {
	clearTape();
    }
    
    /* returns true if this list is empty, otherwise false */
    public boolean isEmpty() {
	return (this.leftEnd == null);
    }
    
    /* returns true, if this list contains Object o, otherwise false */
    public boolean search(Object o) {
	Enumeration en = this.enumerate();
	while (en.hasMoreElements()) {
	    if (en.nextElement().equals(o))
		return true;
	}
	return false;
    }
    
    /* inserts a TapeNode that contains Object o at the leftEnd*/
    public void insertLeftEnd(Object o) {
	TapeNode tmp = this.leftEnd;
	this.leftEnd = new TapeNode(o, null, tmp, reversed);
	if (tmp != null) {
	    tmp.setPreviousNode(this.leftEnd, reversed);
	} else {
	    this.rightEnd = this.leftEnd;
	}
    }

    /* inserts a TapeNode that contains Object o at the rightEnd*/
    public void insertRightEnd(Object o) {
	reverseList();
	insertLeftEnd(o);
	reverseList();
    }
  
    /* remove the specified TapeNode */
    private void removeNode(TapeNode node) {
	TapeNode predecessor = node.getPreviousNode(reversed);
	TapeNode successor = node.getNextNode(reversed);
	if (predecessor == null) {
	    this.leftEnd = successor;
	    if (successor == null) {
		this.rightEnd = predecessor;
	    } else {
		successor.setPreviousNode(null, reversed);
	    }
	} else {
	    if (successor == null) {
		this.rightEnd = predecessor;
		predecessor.setNextNode(null, reversed);
	    } else {
		predecessor.setNextNode(successor, reversed);
		successor.setPreviousNode(predecessor, reversed);
	    }
	}
    }
    
    /* removes the first TapeNode that contains Object o */
    public void remove(Object o) {
	TapeNode tmp = this.leftEnd;
	while (tmp != null) {
	    if (tmp.getContent().equals(o)) {
		removeNode(tmp);
	    }
	    tmp = tmp.getNextNode(reversed);
	}
    }
      
    /* reverses the list */
    public void reverseList() {
	TapeNode tmp = this.leftEnd;
	this.leftEnd = this.rightEnd;
	this.rightEnd = tmp;
	if (reversed) {
	    reversed = false;
	} else {
	    reversed = true;
	}
    }

    /* Clears the tape */
    public void clearTape() {
	this.leftEnd = null;
	this.rightEnd = null;
	this.current = null;
	this.reversed = false;
    }
    
    
    /* needed in order to implement the interface Enumeration */
    public Enumeration enumerate() {
	return new TapeEnumeration(this);
    }
            
    /* returns the list's content as a string object */
    public String toString() {
	String str = "";
	Enumeration en = this.enumerate();
	while (en.hasMoreElements()) {
	    str += en.nextElement().toString() + " ";
	}
	return str;
    }
}


/* implements the interface Enumeration */
class TapeEnumeration implements Enumeration {

    Tape list;
    TapeNode enumPointer;
    boolean reversed;
    
    public TapeEnumeration(Tape list) {
	this.list = list;
	this.enumPointer = list.leftEnd;
	this.reversed = list.reversed;
    }

    public boolean hasMoreElements() {
	return (enumPointer != null);
    }

    public Object nextElement() {
	TapeNode tmp = enumPointer;
	enumPointer  = tmp.getNextNode(reversed);
	return tmp.getContent();
    }
}
